"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const severity_level_1 = require("../severity-level");
const base_payload_1 = require("./base-payload");
const utils_1 = require("./utils");
class ObjectPayload extends base_payload_1.BasePayload {
    constructor() {
        super(...arguments);
        this.entries = new Map();
    }
    getKind() {
        return 'object';
    }
    buildFromSchema(rawSchema) {
        for (const keyEntry of rawSchema['$_terms'].keys) {
            this.addEntry(keyEntry.key, utils_1.generatePayload(keyEntry.schema));
        }
        return this;
    }
    addEntry(key, value) {
        this.entries.set(key, value);
    }
    generateMock() {
        const targetObject = {};
        for (const [keyName, value] of this.entries) {
            targetObject[keyName] = value.generateMock();
        }
        return targetObject;
    }
    generateAttacks() {
        const validObject = this.generateMock();
        const payloads = [];
        for (const [keyName, value] of this.entries) {
            const attacks = value.generateAttacks({ keyName });
            // TODO Attack ideas & discussions
            // - This is a JS validator, PHP payloads and other should be avoided
            // - Try to prepend/append data to regex due to lack of "$^"
            // - Detect malicious regex that can cause DDOS
            // - Include attacks in Base64 content
            // - Possibilities for injections in 'dataUri' (malicious SVG)
            // - Try to 'brute-force' strings (alphanum, emails, ...)
            // - Look for .git URL
            // - Try to detect Joi unsafe number use
            // - Key name agnostic usage
            //      * Key match path/file/upload: try path traversal, JS nullbyte files
            //      * Key match user/name: try to detect lack of trim() usage for 'admin '
            //      * Key match phone/mobile/line: paid numbers, ...
            for (const attack of attacks) {
                const attackPayload = Object.assign({}, validObject);
                attackPayload[keyName] = attack.payload;
                payloads.push({
                    severity: attack.severity,
                    messages: attack.messages,
                    payload: attackPayload,
                    tags: attack.tags
                });
            }
        }
        payloads.push({
            severity: severity_level_1.SeverityLevel.LOW,
            messages: ['Injecting unknown key with malicious values, may have side effects'],
            payload: Object.assign(Object.assign({}, validObject), { unknownKey: '<script></script>' })
        });
        return payloads;
    }
}
exports.ObjectPayload = ObjectPayload;
