"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const RandExp = require("randexp");
const attack_1 = require("../attack");
const base_payload_1 = require("./base-payload");
var StringConstraint;
(function (StringConstraint) {
    // Case constraints
    StringConstraint["case"] = "case";
    // Length constraints
    StringConstraint["length"] = "length";
    StringConstraint["min"] = "min";
    StringConstraint["max"] = "max";
    // Pattern constraints
    StringConstraint["base64"] = "base64";
    StringConstraint["dataUri"] = "dataUri";
    StringConstraint["domain"] = "domain";
    StringConstraint["email"] = "email";
    StringConstraint["hostname"] = "hostname";
    StringConstraint["ip"] = "ip";
    StringConstraint["pattern"] = "pattern";
    StringConstraint["uri"] = "uri";
    // Blacklisted constraints
    StringConstraint["alphanum"] = "alphanum";
    StringConstraint["creditCard"] = "creditCard";
    StringConstraint["guid"] = "guid";
    StringConstraint["hex"] = "hex";
    StringConstraint["isoDate"] = "isoDate";
    StringConstraint["isoDuration"] = "isoDuration";
    StringConstraint["token"] = "token";
})(StringConstraint || (StringConstraint = {}));
class StringPayload extends base_payload_1.BasePayload {
    getKind() {
        return 'string';
    }
    buildFromSchema(rawSchema) {
        if (!rawSchema._rules) {
            return this;
        }
        const allowedConstraints = Object.values(StringConstraint);
        for (const stringRule of rawSchema._rules) {
            if (!allowedConstraints.includes(stringRule.name)) {
                continue;
            }
            if (stringRule.name === StringConstraint.min) {
                this.addConstraint({ name: StringConstraint.min, details: { limit: stringRule.args.limit } });
            }
            else if (stringRule.name === StringConstraint.max) {
                this.addConstraint({ name: StringConstraint.max, details: { limit: stringRule.args.limit } });
            }
            else if (stringRule.name === StringConstraint.length) {
                this.addConstraint({ name: StringConstraint.length, details: { limit: stringRule.args.limit } });
            }
            else if (stringRule.name === StringConstraint.pattern || stringRule.name === 'regex') {
                this.addConstraint({ name: StringConstraint.pattern, details: { regex: stringRule.args.regex } });
            }
            else {
                this.addConstraint({ name: stringRule.name });
            }
        }
        return this;
    }
    generateMock() {
        if (this.hasConstraint(StringConstraint.ip)) {
            return '37.56.180.29';
        }
        if (this.hasConstraint(StringConstraint.creditCard)) {
            return '5500 0000 0000 0004';
        }
        if (this.hasConstraint(StringConstraint.guid)) {
            return '75442486-0878-440c-9db1-a7006c25a39f';
        }
        if (this.hasConstraint(StringConstraint.domain)) {
            return 'example.domain.com';
        }
        if (this.hasConstraint(StringConstraint.email)) {
            return 'john.doe@company.com';
        }
        if (this.hasConstraint(StringConstraint.pattern)) {
            return new RandExp(this.getConstraint(StringConstraint.pattern).regex).gen();
        }
        if (this.hasConstraint(StringConstraint.uri)) {
            return 'https://domain.com';
        }
        // From now, we should be dealing with 'loose validation' strings such
        // as alphanum, token or hex. We will thus focus on enforcing length and
        // case rules.
        let baseString = 'a';
        if (this.hasConstraint(StringConstraint.length)) {
            const expectedLength = this.getConstraint(StringConstraint.length).limit;
            baseString = _.repeat(baseString, expectedLength);
        }
        else if (this.hasConstraint(StringConstraint.min)) {
            const expectedMinLength = this.getConstraint(StringConstraint.min).limit;
            baseString = _.repeat(baseString, expectedMinLength);
        }
        // This string should pass the alphanum, token & hex format validation
        return baseString;
    }
    generateAttacks(options) {
        var _a, _b;
        if (this.hasConstraint(StringConstraint.email)) {
            return attack_1.STRING_ATTACKS.EMAIL;
        }
        if (this.hasConstraint(StringConstraint.ip)) {
            return attack_1.STRING_ATTACKS.IP;
        }
        if (this.hasConstraint(StringConstraint.hostname)) {
            return attack_1.STRING_ATTACKS.HOSTNAME;
        }
        if (this.hasConstraint(StringConstraint.uri)) {
            return attack_1.STRING_ATTACKS.URI;
        }
        let baseCollection = attack_1.STRING_ATTACKS.COMMON;
        // If we are dealing with a potential file path input, we can add a new set
        // of dedicated path traversal attacks.
        if ((_a = options === null || options === void 0 ? void 0 : options.keyName) === null || _a === void 0 ? void 0 : _a.match(/(path|file|doc|image|upload)/i)) {
            baseCollection = [
                ...attack_1.STRING_ATTACKS.PATH_TRAVERSAL,
                ...baseCollection
            ];
        }
        // If we are dealing with a potential phone numbers, we may add another
        // range of attacks linked to mobile phones.
        if ((_b = options === null || options === void 0 ? void 0 : options.keyName) === null || _b === void 0 ? void 0 : _b.match(/(phone|line|contact|mobile)/i)) {
            baseCollection = [
                ...attack_1.STRING_ATTACKS.PHONE,
                ...baseCollection
            ];
        }
        return baseCollection;
    }
}
exports.StringPayload = StringPayload;
